/**********************************************************************************
 *   ********** Computer and Real-Time Group . University of Cantabria **********
 *                         *** SwiftEagle  Project ***   
 *   
 *   Client active class: MissileController
 *   Clase que implementa un ejemplo de cliente controlador de un elemento al que
 *   se accede a travs de la interfaz remota Missile. En este ejemplo el controlador
 *   Hace subir al misil casi verticalmente (direccin {1,1,10}), monitorizando la
 *   salida y al llegar a los 1000 metros los hace explotar.
 *      
 *    Author: J.M. Drake
 *    Version: 08-05-2013
 *
 **********************************************************************************/

import java.rmi.registry.*;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.*;

public class MissileController {
	// Variable esttica que gestiona el observador TargetObserver. Cuando el missil
	// alcanza la atura propuesta como target la pone a true.
	public static boolean asCloseToTarget=false;
	
	public static void main(String[] args) {

		String missileName="Misil7831";
        String elHost="127.0.0.1";

        // Se gestionan diferentes exceptios, del RMI (RemoteException) y de la aplicaci�n
        // (por ejemplo UnreadyException).
        try {
        	 // Se crea el objeto de callback  como un RemoteObject ya registrado en
     	     // en RMI (ver clase TargetObserver)
        	 Observer observer=(Observer)new TargetObserver();
        	 // Se genera aleatoriamente un password long.
        	 long thePassword=(long)(Long.MAX_VALUE*Math.random());
        	 // Se localiza el Registri en el computador que se ha pasado por argumento
        	 // de main() y en el puerto estndar 1099
             Registry registry = LocateRegistry.getRegistry(elHost);
             // Se localiza el misil en el registry con el nombre pasado como argumento
             // en main()
             Missile theMissile = (Missile) registry.lookup(missileName); 
             
             //*** Comienza el control del missil ***//
             // Se lanza el missil desde el angar {0,0,0} y se le pasa el password
             theMissile.shot(new Position(0,0,0),  thePassword);
             // Se le ordena que se desplaze en la direccin {1 1 10}
             theMissile.setDirection(new Direction(1,1,10));
             // Se pasa el callback para que pueda avisar cuando alcance la posicin
             // mas prsima  a las  corrdenadas {0,0,1000}
             theMissile.nearnessReport(new Position(0,0,1000), observer);
             // Mientras que con el callback no se haya establecido asCloseToTarget a true
             while(!asCloseToTarget){
            	 //Cada 500 ms
            	 Thread.sleep(500);
            	 // Monitoriza la posicin que alcanza el missil
            	 Position pos=theMissile.getPosition();
            	 System.out.println("current position: "+pos.x+"   "+pos.y+"   "+pos.z);
             }
             System.out.println("Punto prximo al objetivo");
             // Ya ha llegado al punto mas proximo al target. Se le ordena explotar,
             // pero si el password no es el correcto se lanza la excepcin UnknownException
             // y no explota sino que se desactiva
             try{theMissile.explode( thePassword);}catch(UnknownException u) {
            	 System.out.println("Eres un intruso y no exploto");
            	 theMissile.turnOff();
             }
             // se da de baja el objeto remoto del callback en el RMI.
             try {
     			
     			UnicastRemoteObject.unexportObject(observer, false);
     		} catch (NoSuchObjectException e) {}

       } catch (Exception e) {
              System.err.println("Excepcin del cliente: " + e.toString());
       }
        // Termina
	}
}
